@@ -36,7 +36,7 @@ class FunctionSummary(AbstractPrinter):
             if c.is_top_level:
                 continue
             (name, inheritance, var, func_summaries, modif_summaries) = c.get_summary()
-            txt = "\nContract %s" % name
+            txt = f"\nContract {name}"
             txt += "\nContract vars: " + str(var)
             txt += "\nInheritance:: " + str(inheritance)
             table = MyPrettyTable(
