@@ -63,14 +63,14 @@ class TmpCall(OperationWithLValue):  # pylint: disable=too-many-instance-attribu
     def call_id(self):
         return self._callid
 
-    @property
-    def read(self):
-        return [self.called]
-
     @call_id.setter
     def call_id(self, c):
         self._callid = c
 
+    @property
+    def read(self):
+        return [self.called]
+
     @property
     def called(self):
         return self._called
@@ -95,4 +95,4 @@ class TmpCall(OperationWithLValue):  # pylint: disable=too-many-instance-attribu
         self._ori = ori
 
     def __str__(self):
-        return str(self.lvalue) + " = TMPCALL{} ".format(self.nbr_arguments) + str(self._called)
+        return str(self.lvalue) + f" = TMPCALL{self.nbr_arguments} " + str(self._called)
