@@ -1,13 +1,17 @@
+import argparse
 import json
 import os
 import re
 import logging
 from collections import defaultdict
+from typing import Dict, List, Type, Union
+
 from crytic_compile.cryticparser.defaults import (
     DEFAULTS_FLAG_IN_CONFIG as DEFAULTS_FLAG_IN_CONFIG_CRYTIC_COMPILE,
 )
 
-from slither.detectors.abstract_detector import classification_txt
+from slither.detectors.abstract_detector import classification_txt, AbstractDetector
+from slither.printers.abstract_printer import AbstractPrinter
 from slither.utils.colors import yellow, red
 from slither.utils.myprettytable import MyPrettyTable
 
@@ -34,6 +38,10 @@ defaults_flag_in_config = {
     "exclude_low": False,
     "exclude_medium": False,
     "exclude_high": False,
+    "fail_pedantic": True,
+    "fail_low": False,
+    "fail_medium": False,
+    "fail_high": False,
     "json": None,
     "sarif": None,
     "json-types": ",".join(DEFAULT_JSON_OUTPUT_TYPES),
@@ -43,7 +51,6 @@ defaults_flag_in_config = {
     # debug command
     "skip_assembly": False,
     "legacy_ast": False,
-    "ignore_return_value": False,
     "zip": None,
     "zip_type": "lzma",
     "show_ignored_findings": False,
@@ -51,32 +58,44 @@ defaults_flag_in_config = {
 }
 
 
-def read_config_file(args):
+def read_config_file(args: argparse.Namespace) -> None:
+    # No config file was provided as an argument
+    if args.config_file is None:
+        # Check wether the default config file is present
+        if os.path.exists("slither.config.json"):
+            # The default file exists, use it
+            args.config_file = "slither.config.json"
+        else:
+            return
+
     if os.path.isfile(args.config_file):
         try:
-            with open(args.config_file) as f:
+            with open(args.config_file, encoding="utf8") as f:
                 config = json.load(f)
                 for key, elem in config.items():
                     if key not in defaults_flag_in_config:
                         logger.info(
-                            yellow(
-                                "{} has an unknown key: {} : {}".format(args.config_file, key, elem)
-                            )
+                            yellow(f"{args.config_file} has an unknown key: {key} : {elem}")
                         )
                         continue
                     if getattr(args, key) == defaults_flag_in_config[key]:
                         setattr(args, key, elem)
         except json.decoder.JSONDecodeError as e:
-            logger.error(
-                red("Impossible to read {}, please check the file {}".format(args.config_file, e))
-            )
+            logger.error(red(f"Impossible to read {args.config_file}, please check the file {e}"))
+    else:
+        logger.error(red(f"File {args.config_file} is not a file or does not exist"))
+        logger.error(yellow("Falling back to the default settings..."))
 
 
-def output_to_markdown(detector_classes, printer_classes, filter_wiki):
-    def extract_help(cls):
+def output_to_markdown(
+    detector_classes: List[Type[AbstractDetector]],
+    printer_classes: List[Type[AbstractPrinter]],
+    filter_wiki: str,
+) -> None:
+    def extract_help(cls: Union[Type[AbstractDetector], Type[AbstractPrinter]]) -> str:
         if cls.WIKI == "":
             return cls.HELP
-        return "[{}]({})".format(cls.HELP, cls.WIKI)
+        return f"[{cls.HELP}]({cls.WIKI})"
 
     detectors_list = []
     print(filter_wiki)
@@ -98,11 +117,7 @@ def output_to_markdown(detector_classes, printer_classes, filter_wiki):
     )
     idx = 1
     for (argument, help_info, impact, confidence) in detectors_list:
-        print(
-            "{} | `{}` | {} | {} | {}".format(
-                idx, argument, help_info, classification_txt[impact], confidence
-            )
-        )
+        print(f"{idx} | `{argument}` | {help_info} | {classification_txt[impact]} | {confidence}")
         idx = idx + 1
 
     print()
@@ -116,11 +131,11 @@ def output_to_markdown(detector_classes, printer_classes, filter_wiki):
     printers_list = sorted(printers_list, key=lambda element: (element[0]))
     idx = 1
     for (argument, help_info) in printers_list:
-        print("{} | `{}` | {}".format(idx, argument, help_info))
+        print(f"{idx} | `{argument}` | {help_info}")
         idx = idx + 1
 
 
-def get_level(l):
+def get_level(l: str) -> int:
     tab = l.count("\t") + 1
     if l.replace("\t", "").startswith(" -"):
         tab = tab + 1
@@ -129,7 +144,7 @@ def get_level(l):
     return tab
 
 
-def convert_result_to_markdown(txt):
+def convert_result_to_markdown(txt: str) -> str:
     # -1 to remove the last \n
     lines = txt[0:-1].split("\n")
     ret = []
@@ -147,16 +162,21 @@ def convert_result_to_markdown(txt):
     return "".join(ret)
 
 
-def output_results_to_markdown(all_results, checklistlimit: str):
+def output_results_to_markdown(all_results: List[Dict], checklistlimit: str) -> None:
     checks = defaultdict(list)
-    info = defaultdict(dict)
-    for results in all_results:
-        checks[results["check"]].append(results)
-        info[results["check"]] = {"impact": results["impact"], "confidence": results["confidence"]}
+    info: Dict = defaultdict(dict)
+    for results_ in all_results:
+        checks[results_["check"]].append(results_)
+        info[results_["check"]] = {
+            "impact": results_["impact"],
+            "confidence": results_["confidence"],
+        }
 
     print("Summary")
-    for check in checks:
-        print(f" - [{check}](#{check}) ({len(checks[check])} results) ({info[check]['impact']})")
+    for check_ in checks:
+        print(
+            f" - [{check_}](#{check_}) ({len(checks[check_])} results) ({info[check_]['impact']})"
+        )
 
     counter = 0
     for (check, results) in checks.items():
@@ -178,8 +198,7 @@ def output_results_to_markdown(all_results, checklistlimit: str):
             print(f"**More results were found, check [{checklistlimit}]({checklistlimit})**")
 
 
-def output_wiki(detector_classes, filter_wiki):
-    detectors_list = []
+def output_wiki(detector_classes: List[Type[AbstractDetector]], filter_wiki: str) -> None:
 
     # Sort by impact, confidence, and name
     detectors_list = sorted(
@@ -202,11 +221,11 @@ def output_wiki(detector_classes, filter_wiki):
         exploit_scenario = detector.WIKI_EXPLOIT_SCENARIO
         recommendation = detector.WIKI_RECOMMENDATION
 
-        print("\n## {}".format(title))
+        print(f"\n## {title}")
         print("### Configuration")
-        print("* Check: `{}`".format(check))
-        print("* Severity: `{}`".format(impact))
-        print("* Confidence: `{}`".format(confidence))
+        print(f"* Check: `{check}`")
+        print(f"* Severity: `{impact}`")
+        print(f"* Confidence: `{confidence}`")
         print("\n### Description")
         print(description)
         if exploit_scenario:
@@ -216,7 +235,7 @@ def output_wiki(detector_classes, filter_wiki):
         print(recommendation)
 
 
-def output_detectors(detector_classes):
+def output_detectors(detector_classes: List[Type[AbstractDetector]]) -> None:
     detectors_list = []
     for detector in detector_classes:
         argument = detector.ARGUMENT
@@ -235,12 +254,15 @@ def output_detectors(detector_classes):
     )
     idx = 1
     for (argument, help_info, impact, confidence) in detectors_list:
-        table.add_row([idx, argument, help_info, classification_txt[impact], confidence])
+        table.add_row([str(idx), argument, help_info, classification_txt[impact], confidence])
         idx = idx + 1
     print(table)
 
 
-def output_detectors_json(detector_classes):  # pylint: disable=too-many-locals
+# pylint: disable=too-many-locals
+def output_detectors_json(
+    detector_classes: List[Type[AbstractDetector]],
+) -> List[Dict]:
     detectors_list = []
     for detector in detector_classes:
         argument = detector.ARGUMENT
@@ -300,7 +322,7 @@ def output_detectors_json(detector_classes):  # pylint: disable=too-many-locals
     return table
 
 
-def output_printers(printer_classes):
+def output_printers(printer_classes: List[Type[AbstractPrinter]]) -> None:
     printers_list = []
     for printer in printer_classes:
         argument = printer.ARGUMENT
@@ -312,12 +334,12 @@ def output_printers(printer_classes):
     printers_list = sorted(printers_list, key=lambda element: (element[0]))
     idx = 1
     for (argument, help_info) in printers_list:
-        table.add_row([idx, argument, help_info])
+        table.add_row([str(idx), argument, help_info])
         idx = idx + 1
     print(table)
 
 
-def output_printers_json(printer_classes):
+def output_printers_json(printer_classes: List[Type[AbstractPrinter]]) -> List[Dict]:
     printers_list = []
     for printer in printer_classes:
         argument = printer.ARGUMENT
