@@ -8,6 +8,9 @@ from typing import Type, Optional, List
 import pytest
 from deepdiff import DeepDiff  # pip install deepdiff
 
+from solc_select.solc_select import install_artifacts as install_solc_versions
+from solc_select.solc_select import installed_versions as get_installed_solc_versions
+
 from slither import Slither
 from slither.detectors.abstract_detector import AbstractDetector
 from slither.detectors import all_detectors
@@ -52,7 +55,7 @@ def id_test(test_item: Test):
     return f"{test_item.detector}: {test_item.solc_ver}/{test_item.test_file}"
 
 
-ALL_TESTS = [
+ALL_TEST_OBJECTS = [
     Test(
         all_detectors.UninitializedFunctionPtrsConstructor,
         "uninitialized_function_ptr_constructor.sol",
@@ -113,6 +116,26 @@ ALL_TESTS = [
         "DAO.sol",
         "0.4.25",
     ),
+    Test(
+        all_detectors.ReentrancyReadBeforeWritten,
+        "comment.sol",
+        "0.8.2",
+    ),
+    Test(
+        all_detectors.ReentrancyReadBeforeWritten,
+        "no-reentrancy-staticcall.sol",
+        "0.5.16",
+    ),
+    Test(
+        all_detectors.ReentrancyReadBeforeWritten,
+        "no-reentrancy-staticcall.sol",
+        "0.6.11",
+    ),
+    Test(
+        all_detectors.ReentrancyReadBeforeWritten,
+        "no-reentrancy-staticcall.sol",
+        "0.7.6",
+    ),
     Test(
         all_detectors.BooleanEquality,
         "boolean-constant-equality.sol",
@@ -373,23 +396,23 @@ ALL_TESTS = [
     Test(all_detectors.LockedEther, "locked_ether.sol", "0.6.11"),
     Test(all_detectors.LockedEther, "locked_ether.sol", "0.7.6"),
     Test(
-        all_detectors.ArbitrarySend,
-        "arbitrary_send.sol",
+        all_detectors.ArbitrarySendEth,
+        "arbitrary_send_eth.sol",
         "0.4.25",
     ),
     Test(
-        all_detectors.ArbitrarySend,
-        "arbitrary_send.sol",
+        all_detectors.ArbitrarySendEth,
+        "arbitrary_send_eth.sol",
         "0.5.16",
     ),
     Test(
-        all_detectors.ArbitrarySend,
-        "arbitrary_send.sol",
+        all_detectors.ArbitrarySendEth,
+        "arbitrary_send_eth.sol",
         "0.6.11",
     ),
     Test(
-        all_detectors.ArbitrarySend,
-        "arbitrary_send.sol",
+        all_detectors.ArbitrarySendEth,
+        "arbitrary_send_eth.sol",
         "0.7.6",
     ),
     Test(
@@ -487,6 +510,11 @@ ALL_TESTS = [
         "external_function_2.sol",
         "0.4.25",
     ),
+    Test(
+        all_detectors.ExternalFunction,
+        "external_function_3.sol",
+        "0.4.25",
+    ),
     Test(
         all_detectors.ExternalFunction,
         "external_function.sol",
@@ -497,6 +525,11 @@ ALL_TESTS = [
         "external_function_2.sol",
         "0.5.16",
     ),
+    Test(
+        all_detectors.ExternalFunction,
+        "external_function_3.sol",
+        "0.5.16",
+    ),
     Test(
         all_detectors.ExternalFunction,
         "external_function.sol",
@@ -507,6 +540,11 @@ ALL_TESTS = [
         "external_function_2.sol",
         "0.6.11",
     ),
+    Test(
+        all_detectors.ExternalFunction,
+        "external_function_3.sol",
+        "0.6.11",
+    ),
     Test(
         all_detectors.ExternalFunction,
         "external_function.sol",
@@ -517,6 +555,11 @@ ALL_TESTS = [
         "external_function_2.sol",
         "0.7.6",
     ),
+    Test(
+        all_detectors.ExternalFunction,
+        "external_function_3.sol",
+        "0.7.6",
+    ),
     Test(
         all_detectors.NamingConvention,
         "naming_convention.sol",
@@ -632,6 +675,16 @@ ALL_TESTS = [
         "shadowing_abstract.sol",
         "0.5.16",
     ),
+    Test(
+        all_detectors.ShadowingAbstractDetection,
+        "shadowing_state_variable.sol",
+        "0.7.5",
+    ),
+    Test(
+        all_detectors.ShadowingAbstractDetection,
+        "public_gap_variable.sol",
+        "0.7.5",
+    ),
     Test(
         all_detectors.StateShadowing,
         "shadowing_state_variable.sol",
@@ -647,6 +700,16 @@ ALL_TESTS = [
         "shadowing_state_variable.sol",
         "0.6.11",
     ),
+    Test(
+        all_detectors.StateShadowing,
+        "shadowing_state_variable.sol",
+        "0.7.5",
+    ),
+    Test(
+        all_detectors.StateShadowing,
+        "public_gap_variable.sol",
+        "0.7.5",
+    ),
     Test(
         all_detectors.StateShadowing,
         "shadowing_state_variable.sol",
@@ -721,6 +784,11 @@ ALL_TESTS = [
         "right_to_left_override.sol",
         "0.6.11",
     ),
+    Test(
+        all_detectors.RightToLeftOverride,
+        "unicode_direction_override.sol",
+        "0.8.0",
+    ),
     Test(all_detectors.VoidConstructor, "void-cst.sol", "0.4.25"),
     Test(all_detectors.VoidConstructor, "void-cst.sol", "0.5.16"),
     Test(all_detectors.VoidConstructor, "void-cst.sol", "0.6.11"),
@@ -810,6 +878,11 @@ ALL_TESTS = [
         "Fixed.sol",
         "0.4.25",
     ),
+    Test(
+        all_detectors.UnprotectedUpgradeable,
+        "whitelisted.sol",
+        "0.4.25",
+    ),
     Test(
         all_detectors.UnprotectedUpgradeable,
         "Buggy.sol",
@@ -820,6 +893,11 @@ ALL_TESTS = [
         "Fixed.sol",
         "0.5.16",
     ),
+    Test(
+        all_detectors.UnprotectedUpgradeable,
+        "whitelisted.sol",
+        "0.5.16",
+    ),
     Test(
         all_detectors.UnprotectedUpgradeable,
         "Buggy.sol",
@@ -830,6 +908,11 @@ ALL_TESTS = [
         "Fixed.sol",
         "0.6.11",
     ),
+    Test(
+        all_detectors.UnprotectedUpgradeable,
+        "whitelisted.sol",
+        "0.6.11",
+    ),
     Test(
         all_detectors.UnprotectedUpgradeable,
         "Buggy.sol",
@@ -840,6 +923,26 @@ ALL_TESTS = [
         "Fixed.sol",
         "0.7.6",
     ),
+    Test(
+        all_detectors.UnprotectedUpgradeable,
+        "whitelisted.sol",
+        "0.7.6",
+    ),
+    Test(
+        all_detectors.UnprotectedUpgradeable,
+        "Buggy.sol",
+        "0.8.15",
+    ),
+    Test(
+        all_detectors.UnprotectedUpgradeable,
+        "Fixed.sol",
+        "0.8.15",
+    ),
+    Test(
+        all_detectors.UnprotectedUpgradeable,
+        "whitelisted.sol",
+        "0.8.15",
+    ),
     Test(
         all_detectors.ABIEncoderV2Array,
         "storage_ABIEncoderV2_array.sol",
@@ -1149,6 +1252,122 @@ ALL_TESTS = [
         "write-after-write.sol",
         "0.8.0",
     ),
+    Test(
+        all_detectors.ShiftParameterMixup,
+        "shift_parameter_mixup.sol",
+        "0.4.25",
+    ),
+    Test(
+        all_detectors.ShiftParameterMixup,
+        "shift_parameter_mixup.sol",
+        "0.5.16",
+    ),
+    Test(
+        all_detectors.ShiftParameterMixup,
+        "shift_parameter_mixup.sol",
+        "0.6.11",
+    ),
+    Test(
+        all_detectors.ShiftParameterMixup,
+        "shift_parameter_mixup.sol",
+        "0.7.6",
+    ),
+    Test(
+        all_detectors.MissingInheritance,
+        "unimplemented_interface.sol",
+        "0.4.25",
+    ),
+    Test(
+        all_detectors.MissingInheritance,
+        "unimplemented_interface.sol",
+        "0.5.16",
+    ),
+    Test(
+        all_detectors.MissingInheritance,
+        "unimplemented_interface.sol",
+        "0.6.11",
+    ),
+    Test(
+        all_detectors.MissingInheritance,
+        "unimplemented_interface.sol",
+        "0.7.6",
+    ),
+    # Does not work on the CI. Most likely because of solc 0.4.2?
+    # Test(
+    #     all_detectors.EnumConversion,
+    #     "enum_conversion.sol",
+    #     "0.4.2",
+    # ),
+    Test(
+        all_detectors.MultipleConstructorSchemes,
+        "multiple_constructor_schemes.sol",
+        "0.4.22",
+    ),
+    Test(
+        all_detectors.DeprecatedStandards,
+        "deprecated_calls.sol",
+        "0.4.25",
+    ),
+    Test(
+        all_detectors.DivideBeforeMultiply,
+        "divide_before_multiply.sol",
+        "0.4.25",
+    ),
+    Test(
+        all_detectors.DivideBeforeMultiply,
+        "divide_before_multiply.sol",
+        "0.5.16",
+    ),
+    Test(
+        all_detectors.DivideBeforeMultiply,
+        "divide_before_multiply.sol",
+        "0.6.11",
+    ),
+    Test(
+        all_detectors.DivideBeforeMultiply,
+        "divide_before_multiply.sol",
+        "0.7.6",
+    ),
+    Test(
+        all_detectors.TypeBasedTautology,
+        "type_based_tautology.sol",
+        "0.4.25",
+    ),
+    Test(
+        all_detectors.TypeBasedTautology,
+        "type_based_tautology.sol",
+        "0.5.16",
+    ),
+    Test(
+        all_detectors.TypeBasedTautology,
+        "type_based_tautology.sol",
+        "0.6.11",
+    ),
+    Test(
+        all_detectors.TypeBasedTautology,
+        "type_based_tautology.sol",
+        "0.7.6",
+    ),
+    Test(
+        all_detectors.SimilarVarsDetection,
+        "similar_variables.sol",
+        "0.4.25",
+    ),
+    Test(
+        all_detectors.SimilarVarsDetection,
+        "similar_variables.sol",
+        "0.5.16",
+    ),
+    Test(
+        all_detectors.SimilarVarsDetection,
+        "similar_variables.sol",
+        "0.6.11",
+    ),
+    Test(
+        all_detectors.SimilarVarsDetection,
+        "similar_variables.sol",
+        "0.7.6",
+    ),
     Test(
         all_detectors.MsgValueInLoop,
         "msg_value_loop.sol",
@@ -1199,7 +1418,156 @@ ALL_TESTS = [
         "delegatecall_loop.sol",
         "0.8.0",
     ),
+    Test(
+        all_detectors.ProtectedVariables,
+        "comment.sol",
+        "0.8.2",
+    ),
+    Test(
+        all_detectors.ArbitrarySendErc20NoPermit,
+        "arbitrary_send_erc20.sol",
+        "0.4.25",
+    ),
+    Test(
+        all_detectors.ArbitrarySendErc20NoPermit,
+        "arbitrary_send_erc20.sol",
+        "0.5.16",
+    ),
+    Test(
+        all_detectors.ArbitrarySendErc20NoPermit,
+        "arbitrary_send_erc20.sol",
+        "0.6.11",
+    ),
+    Test(
+        all_detectors.ArbitrarySendErc20NoPermit,
+        "arbitrary_send_erc20.sol",
+        "0.7.6",
+    ),
+    Test(
+        all_detectors.ArbitrarySendErc20NoPermit,
+        "arbitrary_send_erc20.sol",
+        "0.8.0",
+    ),
+    Test(
+        all_detectors.ArbitrarySendErc20NoPermit,
+        "arbitrary_send_erc20_inheritance.sol",
+        "0.8.0",
+    ),
+    Test(
+        all_detectors.ArbitrarySendErc20Permit,
+        "arbitrary_send_erc20_permit.sol",
+        "0.4.25",
+    ),
+    Test(
+        all_detectors.ArbitrarySendErc20Permit,
+        "arbitrary_send_erc20_permit.sol",
+        "0.5.16",
+    ),
+    Test(
+        all_detectors.ArbitrarySendErc20Permit,
+        "arbitrary_send_erc20_permit.sol",
+        "0.6.11",
+    ),
+    Test(
+        all_detectors.ArbitrarySendErc20Permit,
+        "arbitrary_send_erc20_permit.sol",
+        "0.7.6",
+    ),
+    Test(
+        all_detectors.ArbitrarySendErc20Permit,
+        "arbitrary_send_erc20_permit.sol",
+        "0.8.0",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_collision.sol",
+        "0.4.25",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_collision.sol",
+        "0.5.16",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_collision.sol",
+        "0.6.11",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_collision.sol",
+        "0.7.6",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_collision.sol",
+        "0.8.0",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_wrong_return_type.sol",
+        "0.4.25",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_wrong_return_type.sol",
+        "0.5.16",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_wrong_return_type.sol",
+        "0.6.11",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_wrong_return_type.sol",
+        "0.7.6",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_wrong_return_type.sol",
+        "0.8.0",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_state_var_collision.sol",
+        "0.4.25",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_state_var_collision.sol",
+        "0.5.16",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_state_var_collision.sol",
+        "0.6.11",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_state_var_collision.sol",
+        "0.7.6",
+    ),
+    Test(
+        all_detectors.DomainSeparatorCollision,
+        "permit_domain_state_var_collision.sol",
+        "0.8.0",
+    ),
 ]
+
+
+def get_all_tests() -> List[Test]:
+    installed_solcs = set(get_installed_solc_versions())
+    required_solcs = {test.solc_ver for test in ALL_TEST_OBJECTS}
+    missing_solcs = list(required_solcs - installed_solcs)
+    if missing_solcs:
+        install_solc_versions(missing_solcs)
+
+    return ALL_TEST_OBJECTS
+
+
+ALL_TESTS = get_all_tests()
+
 GENERIC_PATH = "/GENERIC_PATH"
 
 
@@ -1227,11 +1595,10 @@ def test_detector(test_item: Test):
 
     for additional_file in test_item.additional_files:
         additional_path = str(pathlib.Path(test_dir_path, additional_file).absolute())
-        results_as_string = results_as_string.replace(
-            additional_path, str(pathlib.Path(GENERIC_PATH))
-        )
-    results_as_string = results_as_string.replace(test_file_path, str(pathlib.Path(GENERIC_PATH)))
-
+        additional_path = additional_path.replace("\\", "\\\\")
+        results_as_string = results_as_string.replace(additional_path, GENERIC_PATH)
+    test_file_path = test_file_path.replace("\\", "\\\\")
+    results_as_string = results_as_string.replace(test_file_path, GENERIC_PATH)
     results = json.loads(results_as_string)
 
     diff = DeepDiff(results, expected_result, ignore_order=True, verbose_level=2)
@@ -1271,16 +1638,16 @@ def _generate_test(test_item: Test, skip_existing=False):
     results = sl.run_detectors()
 
     results_as_string = json.dumps(results)
-    results_as_string = results_as_string.replace(test_file_path, str(pathlib.Path(GENERIC_PATH)))
+    test_file_path = test_file_path.replace("\\", "\\\\")
+    results_as_string = results_as_string.replace(test_file_path, GENERIC_PATH)
 
     for additional_file in test_item.additional_files:
         additional_path = str(pathlib.Path(test_dir_path, additional_file).absolute())
-        results_as_string = results_as_string.replace(
-            additional_path, str(pathlib.Path(GENERIC_PATH))
-        )
+        additional_path = additional_path.replace("\\", "\\\\")
+        results_as_string = results_as_string.replace(additional_path, GENERIC_PATH)
 
     results = json.loads(results_as_string)
-    with open(expected_result_path, "w") as f:
+    with open(expected_result_path, "w", encoding="utf8") as f:
         f.write(json.dumps(results, indent=4))
 
 
